#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2018-11.22
#
# Monitor Check Point IPSec VPN statistics
#
# 2023-05-29: moved gui files to ~/local/lib/check_mk/gui/plugins/...

# sample snmpwalk
# OMD[mysite]:~$ snmpwalk -v2c -c xyz  -ObentU simulant .1.3.6.1.4.1.2620.1.2
# .1.3.6.1.4.1.2620.1.2.1.0 = STRING: IPSec VPN.
# .1.3.6.1.4.1.2620.1.2.2.0 = INTEGER: 6
# .1.3.6.1.4.1.2620.1.2.3.0 = INTEGER: 0
# .1.3.6.1.4.1.2620.1.2.4.1.1.0 = STRING: 43225
# .1.3.6.1.4.1.2620.1.2.4.1.2.0 = STRING: 54672
# .1.3.6.1.4.1.2620.1.2.4.2.1.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.4.2.2.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.4.2.3.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.4.2.4.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.5.2.1.0 = STRING: 11
# .1.3.6.1.4.1.2620.1.2.5.2.2.0 = STRING: 23
# .1.3.6.1.4.1.2620.1.2.5.2.3.0 = STRING: 10
# .1.3.6.1.4.1.2620.1.2.5.2.4.0 = STRING: 128
# .1.3.6.1.4.1.2620.1.2.5.2.9.0 = STRING: 16
# .1.3.6.1.4.1.2620.1.2.5.2.10.0 = STRING: 12
# .1.3.6.1.4.1.2620.1.2.5.3.1.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.5.3.2.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.5.3.3.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.5.3.4.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.5.3.5.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.5.3.6.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.5.3.7.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.5.4.1.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.5.4.2.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.5.4.5.0 = STRING: 43225
# .1.3.6.1.4.1.2620.1.2.5.4.6.0 = STRING: 54672
# .1.3.6.1.4.1.2620.1.2.5.4.7.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.5.4.8.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.5.4.9.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.5.4.10.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.5.4.11.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.5.4.12.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.5.4.13.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.5.4.14.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.5.4.15.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.5.4.16.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.5.4.17.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.5.4.18.0 = STRING: 0
# .1.3.6.1.4.1.2620.1.2.5.4.19.0 = STRING: 8577840
# .1.3.6.1.4.1.2620.1.2.5.4.20.0 = STRING: 3586064
# .1.3.6.1.4.1.2620.1.2.5.4.21.0 = STRING: "4"
# .1.3.6.1.4.1.2620.1.2.5.4.22.0 = STRING: "4"
# .1.3.6.1.4.1.2620.1.2.5.4.23.0 = STRING: "0"
# .1.3.6.1.4.1.2620.1.2.5.4.24.0 = STRING: "0"
# .1.3.6.1.4.1.2620.1.2.5.4.25.0 = STRING: "0"
#
# Sample info
# [[['IPSec VPN\x00', '6', '0', '991140008']],
#  [['43225', '54672', '0', '0', '0', '0']],
#  [['11', '10']],
#  [['0', '0', '0', '0', '0', '0', '0']],
#  [['0', '0', '43225', '54672', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '8577840', '3586064', '4', '4', '0', '0', '0']]
# ]
#
# sample info no IPSec VPN
# [[], [], [], [], []]
#
#
import time
from dataclasses import dataclass
from typing import List, Optional

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    Result,
    State,
    SNMPTree,
    all_of,
    startswith,
    any_of,
    equals,
    Metric,
    check_levels,
    get_value_store,
    get_rate,
    GetRateError,
    IgnoreResultsError,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)


@dataclass
class CheckpointIpsecVpn:
    version: str
    build: str
    packets_enc: int
    packets_dec: int
    esp_sa_in: int
    esp_sa_out: int
    metrics_rate: List
    metrics_count: List


def parse_checkpoint_ipsecvpn(string_table: List[StringTable]) -> Optional[CheckpointIpsecVpn]:
    vpn, cpvgeneral, cpvsastatistics, cpvsaerrors, cpvipsecstatistics, cpvike = string_table
    
    cpvprodname, cpvvermajor, cpvverminor, cpvbuild = vpn[0]
    
    cpvencpackets, cpvdecpackets, cpverrout, cpverrin, cpverrike, cpverrpolicy = cpvgeneral[0]
    
    try:
        cpvcurrespsasin, cpvcurrespsasout = cpvsastatistics[0]
    except IndexError:
        return
    
    cpvsadecrerr, cpvsaautherr, cpvsareplayerr, cpvsapolicyerr, cpvsaothererrin, cpvsaothererrout, \
    cpvsaunknownspierr = cpvsaerrors[0]
    
    cpvipsecudpespencpkts, cpvipsecudpespdecpkts, cpvipsecespencpkts, cpvipsecespdecpkts, cpvipsecdecomprbytesbefore, \
    cpvipsecdecomprbytesafter, cpvipsecdecomproverhead, cpvipsecdecomprpkts, cpvipsecdecomprerr, \
    cpvipseccomprbytesbefore, cpvipseccomprbytesafter, cpvipseccomproverhead, cpvipsecnoncompressiblebytes, \
    cpvipseccompressiblepkts, cpvipsecnoncompressiblepkts, cpvipseccomprerrors, cpvipsecespencbytes, \
    cpvipsecespdecbytes = cpvipsecstatistics[0]
    
    cpvikecurrsas, cpvikecurrinitsas, cpvikecurrrespsas, cpviketotalfailuresinit, cpvikenoresp, \
    cpviketotalfailuresresp = cpvike[0]

    return CheckpointIpsecVpn(
        version=f'{cpvvermajor}.{cpvverminor}',
        build=cpvbuild,
        packets_enc=int(cpvencpackets),
        packets_dec=int(cpvdecpackets),
        esp_sa_in=int(cpvcurrespsasin),
        esp_sa_out=int(cpvcurrespsasout),
        metrics_rate=[
            # cpvsaerrors
            ('cpvsadecrerr', cpvsadecrerr),
            ('cpvsaautherr', cpvsaautherr),
            ('cpvsareplayerr', cpvsareplayerr),
            ('cpvsapolicyerr', cpvsapolicyerr),
            ('cpvsaothererrin', cpvsaothererrin),
            ('cpvsaothererrout', cpvsaothererrout),
            ('cpvsaunknownspierr', cpvsaunknownspierr),
            
            # cpvipsecstatistics
            ('cpvipsecudpespencpkts', cpvipsecudpespencpkts),
            ('cpvipsecudpespdecpkts', cpvipsecudpespdecpkts),
            ('cpvipsecespencpkts', cpvipsecespencpkts),
            ('cpvipsecespdecpkts', cpvipsecespdecpkts),
            ('cpvipsecdecomprbytesbefore', cpvipsecdecomprbytesbefore),
            ('cpvipsecdecomprbytesafter', cpvipsecdecomprbytesafter),
            ('cpvipsecdecomproverhead', cpvipsecdecomproverhead),
            ('cpvipsecdecomprpkts', cpvipsecdecomprpkts),
            ('cpvipsecdecomprerr', cpvipsecdecomprerr),
            ('cpvipseccomprbytesbefore', cpvipseccomprbytesbefore),
            ('cpvipseccomprbytesafter', cpvipseccomprbytesafter),
            ('cpvipseccomproverhead', cpvipseccomproverhead),
            ('cpvipsecnoncompressiblebytes', cpvipsecnoncompressiblebytes),
            ('cpvipseccompressiblepkts', cpvipseccompressiblepkts),
            ('cpvipsecnoncompressiblepkts', cpvipsecnoncompressiblepkts),
            ('cpvipseccomprerrors', cpvipseccomprerrors),
            ('cpvipsecespencbytes', cpvipsecespencbytes),
            ('cpvipsecespdecbytes', cpvipsecespdecbytes),

            # cpvgeneral
            ('cpverrout', cpverrout),
            ('cpverrin', cpverrin),
            ('cpverrike', cpverrike),
            ('cpverrpolicy', cpverrpolicy),
        ],
        metrics_count=[
            # cvpike
            ('cpvikecurrsas', cpvikecurrsas),
            ('cpvikecurrinitsas', cpvikecurrinitsas),
            ('cpvikecurrrespsas', cpvikecurrrespsas),
            ('cpviketotalfailuresinit', cpviketotalfailuresinit),
            ('cpvikenoresp', cpvikenoresp),
            ('cpviketotalfailuresresp', cpviketotalfailuresresp),
        ],
    )


def discovery_checkpoint_ipsecvpn(section: CheckpointIpsecVpn) -> DiscoveryResult:
    yield Service()


def check_checkpoint_ipsecvpn(params, section: CheckpointIpsecVpn) -> CheckResult:
    yield Result(state=State.OK, notice=f'Version: {section.version}')
    yield Result(state=State.OK, notice=f'Build: {section.build}')

    now_time = time.time()
    value_store = get_value_store()
    raise_ingore_res = False

    try:
        value = get_rate(
            value_store,
            f'checkpoint_ipsecvpn_cpvdecpackets',
            now_time,
            section.packets_dec,
            raise_overflow=True
        )
    except GetRateError:
        raise_ingore_res = True
        value = 0

    yield from check_levels(
        value=value,
        metric_name='checkpoint_ipsecvpn_cpvdecpackets',
        label='Packets in',
        render_func=lambda v: f'{v:.0f}/s'
    )

    try:
        value = get_rate(
            value_store,
            f'checkpoint_ipsecvpn_cpvencpackets',
            now_time,
            section.packets_enc,
            raise_overflow=True
        )
    except GetRateError:
        raise_ingore_res = True
        value = 0

    yield from check_levels(
        value=value,
        metric_name='checkpoint_ipsecvpn_cpvencpackets',
        label='Packets out',
        render_func=lambda v: f'{v:.0f}/s'
    )

    yield from check_levels(
        value=section.esp_sa_in,
        metric_name='checkpoint_ipsecvpn_cpvcurrespsasin',
        label='ESP SAs in',
        render_func=lambda v: f'{v:.0f}'
    )
    yield from check_levels(
        value=section.esp_sa_out,
        metric_name='checkpoint_ipsecvpn_cpvcurrespsasout',
        label='ESP SAs out',
        render_func=lambda v: f'{v:.0f}'
    )

    for key, value in section.metrics_rate:
        try:
            value = get_rate(value_store, f'checkpoint_ipsecvpn_{key}', now_time, int(value), raise_overflow=True)
        except GetRateError:
            raise_ingore_res = True
            value = 0
        yield Metric(name=f'checkpoint_ipsecvpn_{key}', value=value, boundaries=(0, None))

    if raise_ingore_res:
        raise IgnoreResultsError('Initializing counters')

    for metric, value in section.metrics_rate:
        yield Metric(name=f'checkpoint_ipsecvpn_{metric}', value=int(value))


register.snmp_section(
    name='checkpoint_ipsecvpn',
    parse_function=parse_checkpoint_ipsecvpn,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.2',  # CHECKPOINT-MIB::vpn
            oids=[
                '1',  # cpvProdName
                '2',  # cpvVerMajor
                '3',  # cpvVerMinor
                '11',  # cpvBuild
            ]),
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.2.4',  # CHECKPOINT-MIB::cpvGeneral
            oids=[
                '1.1',  # cpvEncPackets
                '1.2',  # cpvDecPackets
                '2.1',  # cpvErrOut
                '2.2',  # cpvErrIn
                '2.3',  # cpvErrIke
                '2.4',  # cpvErrPolicy
            ]),
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.2.5.2',  # CHECKPOINT-MIB::cpvSaStatistics
            oids=[
                '1',  # cpvCurrEspSAsIn
                '3',  # cpvCurrEspSAsOut
            ]),
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.2.5.3',  # CHECKPOINT-MIB::cpvSaErrors
            oids=[
                '1',  # cpvSaDecrErr
                '2',  # cpvSaAuthErr
                '3',  # cpvSaReplayErr
                '4',  # cpvSaPolicyErr
                '5',  # cpvSaOtherErrIn
                '6',  # cpvSaOtherErrOut
                '7',  # cpvSaUnknownSpiErr
            ]),
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.2.5.4',  # CHECKPOINT-MIB::cpvIpsecStatistics
            oids=[
                '1',  # cpvIpsecUdpEspEncPkts
                '2',  # cpvIpsecUdpEspDecPkts
                '5',  # cpvIpsecEspEncPkts
                '6',  # cpvIpsecEspDecPkts
                '7',  # cpvIpsecDecomprBytesBefore
                '8',  # cpvIpsecDecomprBytesAfter
                '9',  # cpvIpsecDecomprOverhead
                '10',  # cpvIpsecDecomprPkts
                '11',  # cpvIpsecDecomprErr
                '12',  # cpvIpsecComprBytesBefore
                '13',  # cpvIpsecComprBytesAfter
                '14',  # cpvIpsecComprOverhead
                '15',  # cpvIpsecNonCompressibleBytes
                '16',  # cpvIpsecCompressiblePkts
                '17',  # cpvIpsecNonCompressiblePkts
                '18',  # cpvIpsecComprErrors
                '19',  # cpvIpsecEspEncBytes
                '20',  # cpvIpsecEspDecBytes
            ]),
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.2.9',  # CHECKPOINT-MIB::cpvIKE
            oids=[
                '1.1',  # cpvIKECurrSAs
                '1.2',  # cpvIKECurrInitSAs
                '1.3',  # cpvIKECurrRespSAs
                '2.1',  # cpvIKETotalFailuresInit
                '2.2',  # cpvIKENoResp
                '2.3',  # cpvIKETotalFailuresResp
            ]),
    ],
    detect=all_of(
        any_of(
            startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
            all_of(
                equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
                equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
            )
        ),
        startswith('.1.3.6.1.4.1.2620.1.2.1.0', 'IPSec VPN')
    )
)

register.check_plugin(
    name='checkpoint_ipsecvpn',
    service_name='IPSec VPN',
    discovery_function=discovery_checkpoint_ipsecvpn,
    check_function=check_checkpoint_ipsecvpn,
    check_ruleset_name='checkpoint_ipsecvpn',
    check_default_parameters={},
)
