#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2018-11-22
#
# Check Point Firewall ipsecvpn metrics plugins
# checkpoint_ipsecvpn
#
from cmk.gui.i18n import _

from cmk.gui.plugins.metrics.utils import (
    metric_info,
    graph_info,
    perfometer_info,
)

# cpvsastatistics
metric_info['checkpoint_ipsecvpn_cpvcurrespsasin'] = {
    'title': _('Current ESP SAs in'),
    'unit': 'count',
    'color': '26/a',
}
metric_info['checkpoint_ipsecvpn_cpvcurrespsasout'] = {
    'title': _('Current ESP SAs out'),
    'unit': 'count',
    'color': '16/a',
}

# cpvgeneral
metric_info['checkpoint_ipsecvpn_cpvdecpackets'] = {
    'title': _('Packets/s in'),
    'unit': '1/s',
    'color': '11/a',
}
metric_info['checkpoint_ipsecvpn_cpvencpackets'] = {
    'title': _('Packets/s out'),
    'unit': '1/s',
    'color': '12/a',
}
metric_info['checkpoint_ipsecvpn_cpverrike'] = {
    'title': _('IKE errors/s'),
    'unit': '1/s',
    'color': '11/a',
}
metric_info['checkpoint_ipsecvpn_cpverrin'] = {
    'title': _('in errors/s'),
    'unit': '1/s',
    'color': '12/a',
}
metric_info['checkpoint_ipsecvpn_cpverrout'] = {
    'title': _('out errors/s'),
    'unit': '1/s',
    'color': '13/a',
}
metric_info['checkpoint_ipsecvpn_cpverrpolicy'] = {
    'title': _('policy errors/s'),
    'unit': '1/s',
    'color': '14/a',
}

# cpvsaerrors
metric_info['checkpoint_ipsecvpn_cpvsaautherr'] = {
    'title': _('Authentication errors/s'),
    'unit': '1/s',
    'color': '15/a',
}
metric_info['checkpoint_ipsecvpn_cpvsadecrerr'] = {
    'title': _('SA encryption errors/s'),
    'unit': '1/s',
    'color': '16/a',
}
metric_info['checkpoint_ipsecvpn_cpvsaothererrin'] = {
    'title': _('SA other in errors/s'),
    'unit': '1/s',
    'color': '21/a',
}
metric_info['checkpoint_ipsecvpn_cpvsaothererrout'] = {
    'title': _('SA other out errors/s'),
    'unit': '1/s',
    'color': '22/a',
}
metric_info['checkpoint_ipsecvpn_cpvsapolicyerr'] = {
    'title': _('SA policy errors/s'),
    'unit': '1/s',
    'color': '23/a',
}
metric_info['checkpoint_ipsecvpn_cpvsareplayerr'] = {
    'title': _('SA replay errors/s'),
    'unit': '1/s',
    'color': '24/a',
}
metric_info['checkpoint_ipsecvpn_cpvsaunknownspierr'] = {
    'title': _('SA unknown SPI errors/s'),
    'unit': '1/s',
    'color': '25/a',
}

# cpvipsecstatistics
metric_info['checkpoint_ipsecvpn_cpvipseccomprbytesafter'] = {
    'title': _('bytes after compression/s'),
    'unit': '1/s',
    'color': '11/a',
}
metric_info['checkpoint_ipsecvpn_cpvipseccomprbytesbefore'] = {
    'title': _('bytes before compression/s'),
    'unit': '1/s',
    'color': '12/a',
}
metric_info['checkpoint_ipsecvpn_cpvipseccomprerrors'] = {
    'title': _('compression errors/s'),
    'unit': '1/s',
    'color': '13/a',
}
metric_info['checkpoint_ipsecvpn_cpvipseccompressiblepkts'] = {
    'title': _('packes compressed/s'),
    'unit': '1/s',
    'color': '14/a',
}
metric_info['checkpoint_ipsecvpn_cpvipseccomproverhead'] = {
    'title': _('compression overhead/s'),
    'unit': '1/s',
    'color': '15/a',
}

metric_info['checkpoint_ipsecvpn_cpvipsecdecomprbytesafter'] = {
    'title': _('bytes after decompression/s'),
    'unit': '1/s',
    'color': '16/a',
}
metric_info['checkpoint_ipsecvpn_cpvipsecdecomprbytesbefore'] = {
    'title': _('bytes before decompression/s'),
    'unit': '1/s',
    'color': '21/a',
}
metric_info['checkpoint_ipsecvpn_cpvipsecdecomprerr'] = {
    'title': _('decompression errors/s'),
    'unit': '1/s',
    'color': '22/a',
}
metric_info['checkpoint_ipsecvpn_cpvipsecdecomproverhead'] = {
    'title': _('decompression overhead/s'),
    'unit': '1/s',
    'color': '23/a',
}
metric_info['checkpoint_ipsecvpn_cpvipsecdecomprpkts'] = {
    'title': _('packets decompressed/s'),
    'unit': '1/s',
    'color': '24/a',
}

metric_info['checkpoint_ipsecvpn_cpvipsecespdecbytes'] = {
    'title': _('ESP decapsulated bytes/s'),
    'unit': '1/s',
    'color': '11/a',
}
metric_info['checkpoint_ipsecvpn_cpvipsecespencbytes'] = {
    'title': _('ESP encapsulated bytes/s'),
    'unit': '1/s',
    'color': '12/a',
}
metric_info['checkpoint_ipsecvpn_cpvipsecespdecpkts'] = {
    'title': _('ESP decapsulated packets/s'),
    'unit': '1/s',
    'color': '11/a',
}
metric_info['checkpoint_ipsecvpn_cpvipsecespencpkts'] = {
    'title': _('ESP encapsulated packets/s'),
    'unit': '1/s',
    'color': '12/a',
}
metric_info['checkpoint_ipsecvpn_cpvipsecnoncompressiblebytes'] = {
    'title': _('bytes noncompressible/s'),
    'unit': '1/s',
    'color': '13/a',
}
metric_info['checkpoint_ipsecvpn_cpvipsecnoncompressiblepkts'] = {
    'title': _('packets noncompressible/s'),
    'unit': '1/s',
    'color': '14/a',
}
metric_info['checkpoint_ipsecvpn_cpvipsecudpespdecpkts'] = {
    'title': _('ESP UDP decapsulated packets/s'),
    'unit': '1/s',
    'color': '15/a',
}
metric_info['checkpoint_ipsecvpn_cpvipsecudpespencpkts'] = {
    'title': _('ESP UDP encapsulated packets/s'),
    'unit': '1/s',
    'color': '16/a',
}

# cpvike
metric_info['checkpoint_ipsecvpn_cpvikecurrsas'] = {
    'title': _('Current IKE SAs'),
    'unit': 'count',
    'color': '11/a',
}
metric_info['checkpoint_ipsecvpn_cpvikecurrinitsas'] = {
    'title': _('Current IKE SAs init'),
    'unit': 'count',
    'color': '12/a',
}
metric_info['checkpoint_ipsecvpn_cpvikecurrrespsas'] = {
    'title': _('Current IKE SAs response'),
    'unit': 'count',
    'color': '13/a',
}
metric_info['checkpoint_ipsecvpn_cpviketotalfailuresinit'] = {
    'title': _('Total IKE failures init'),
    'unit': 'count',
    'color': '11/a',
}
metric_info['checkpoint_ipsecvpn_cpvikenoresp'] = {
    'title': _('IKE no respnse'),
    'unit': 'count',
    'color': '12/a',
}
metric_info['checkpoint_ipsecvpn_cpviketotalfailuresresp'] = {
    'title': _('Total IKE failures reponse'),
    'unit': 'count',
    'color': '13/a',
}

##############################################################################
#
# how to graph perdata
#
##############################################################################

graph_info['checkpoint_ipsecvpn_cpvike_sas'] = {
    'title': _('Check Point IKE SAs'),
    'metrics': [
        ('checkpoint_ipsecvpn_cpvikecurrinitsas', 'stack'),
        ('checkpoint_ipsecvpn_cpvikecurrrespsas', 'stack'),
        ('checkpoint_ipsecvpn_cpvikecurrsas', 'line'),
    ],
}

graph_info['checkpoint_ipsecvpn_cpvike_errors'] = {
    'title': _('Check Point IKE errors'),
    'metrics': [
        ('checkpoint_ipsecvpn_cpviketotalfailuresinit', 'line'),
        ('checkpoint_ipsecvpn_cpviketotalfailuresresp', 'line'),
        ('checkpoint_ipsecvpn_cpvikenoresp', 'line'),
    ],
}

graph_info['checkpoint_ipsecvpn_sas'] = {
    'title': _('Check Point IPSec SAs'),
    'metrics': [
        ('checkpoint_ipsecvpn_cpvcurrespsasin', 'area'),
        ('checkpoint_ipsecvpn_cpvcurrespsasout', '-area'),
    ],
}

graph_info['checkpoint_ipsecvpn_bytes'] = {
    'title': _('Check Point IPSec packets'),
    'metrics': [
        ('checkpoint_ipsecvpn_cpvdecpackets', 'area'),
        ('checkpoint_ipsecvpn_cpvencpackets', '-area'),
    ],
}

graph_info['checkpoint_ipsecvpn_bytes_compressed'] = {
    'title': _('Check Point IPSec compression bytes'),
    'metrics': [
        ('checkpoint_ipsecvpn_cpvipseccomprbytesbefore', 'area'),
        ('checkpoint_ipsecvpn_cpvipseccomprbytesafter', 'area'),
        ('checkpoint_ipsecvpn_cpvipsecdecomprbytesbefore', '-area'),
        ('checkpoint_ipsecvpn_cpvipsecdecomprbytesafter', '-ares'),
    ],
}

graph_info['checkpoint_ipsecvpn_compresseion'] = {
    'title': _('Check Point IPSec Compression'),
    'metrics': [
        ('checkpoint_ipsecvpn_cpvipseccomprerrors', 'line'),
        ('checkpoint_ipsecvpn_cpvipseccomproverhead', 'line'),
        ('checkpoint_ipsecvpn_cpvipsecdecomprerr', '-line'),
        ('checkpoint_ipsecvpn_cpvipsecdecomproverhead', '-line'),
    ],
}

graph_info['checkpoint_ipsecvpn_packets'] = {
    'title': _('Check Point IPSec packets'),
    'metrics': [
        ('checkpoint_ipsecvpn_cpvipsecespencpkts', 'line'),
        ('checkpoint_ipsecvpn_cpvipsecudpespencpkts', 'line'),
        ('checkpoint_ipsecvpn_cpvipseccompressiblepkts', 'line'),
        ('checkpoint_ipsecvpn_cpvipsecnoncompressiblepkts', 'line'),
        ('checkpoint_ipsecvpn_cpvipsecespdecpkts', '-line'),
        ('checkpoint_ipsecvpn_cpvipsecudpespdecpkts', '-line'),
        ('checkpoint_ipsecvpn_cpvipsecdecomprpkts', '-line'),
    ],
}

graph_info['checkpoint_ipsecvpn_bytes'] = {
    'title': _('Check Point IPSec bytes'),
    'metrics': [
        ('checkpoint_ipsecvpn_cpvipsecespencbytes', 'line'),
        ('checkpoint_ipsecvpn_cpvipsecnoncompressiblebytes', 'line'),
        ('checkpoint_ipsecvpn_cpvipsecespdecbytes', '-line'),
    ],
}

graph_info['checkpoint_ipsecvpn_errors'] = {
    'title': _('Check Point IPSec errors'),
    'metrics': [
        ('checkpoint_ipsecvpn_cpverrike', 'line'),
        ('checkpoint_ipsecvpn_cpverrin', 'line'),
        ('checkpoint_ipsecvpn_cpverrout', 'line'),
        ('checkpoint_ipsecvpn_cpverrpolicy', 'line'),
        ('checkpoint_ipsecvpn_cpvsaautherr', 'line'),
        ('checkpoint_ipsecvpn_cpvsadecrerr', 'line'),
        ('checkpoint_ipsecvpn_cpvsaothererrin', 'line'),
        ('checkpoint_ipsecvpn_cpvsaothererrout', 'line'),
        ('checkpoint_ipsecvpn_cpvsapolicyerr', 'line'),
        ('checkpoint_ipsecvpn_cpvsareplayerr', 'line'),
        ('checkpoint_ipsecvpn_cpvsaunknownspierr', 'line'),
    ],
}

##############################################################################
#
# define perf-o-meter
#
##############################################################################


# perfometer_info['']=('stacked', [
#     {
#         'type': 'logarithmic',
#         'metric': 'checkpoint_connections_connections',
#         'half_value': 100000.0,
#         'exponent': 2,
#     },
#     {
#         'type': 'logarithmic',
#         'metric': 'checkpoint_connections_fwconnections',
#         'half_value': 1000.0,
#         'exponent': 2,
#     },
# ]))
